import { onSocketMessage, sendSocketMessage,  showModal } from "@tarojs/taro";
import { Subject } from "rxjs";
import { useInjector } from "../../common/useInjector";
import { onMounted } from "vue";
import { useAuthStore } from "../auth/useAuthStore";
import { useChannelInfo } from "./useChannelInfo";
import { useState } from "../../common/useState";
import { UserData } from "../../../types/auth";
import { useSocket } from "./useSocket";

interface CallingMessage {
  fromID: string;
  fromName: string;
  toID: string;
  toName:string;
  companyID: string;
  channelID: string;
  msgMainFlag:string;
  msgSubFlag: string;
}


export function useCalling() {
  const [targetUser, setTarget] = useState<UserData>(null)
  const { user } = useInjector(useAuthStore, 'root');
  const channel  = useInjector(useChannelInfo, 'root');
  const ws = useSocket()

  // 初始化websocket
  onMounted( () => {
    ws.init(`wss://www.if-ar.com:3009?fromID=${user.value.id}&fromName=${user.value.nickname}&signID=SA&companyID=${user.value.company_id}`)
    onSocketMessage( res => {
      console.log(res)
      const data = JSON.parse(res.data) 
      if(data.msgMainFlag === 'CallOffer') {
        switch(data.msgSubFlag) {
          case '':
            showModal({
              title: `收到来自${data.fromName}的呼叫，是否同意？`,
              success: () => answer(data, 'Connect'),
              fail: () => answer(data, 'Hangup')
            })
        }
                        
      }
    })  
  })

  // 发出呼叫
  const send = () => {
    channel.createChannel();    
    channel.onReady( () => {
      const data = JSON.stringify({
        fromID: user.value.id,
        fromName: user.value.nickname,
        toID: targetUser.value.id,
        toNmame: targetUser.value.nickname,
        channelID: channel.current.value.channel_id,
        msgMainFlag: 'CallOffer',
        msgSubFlag: 'Request'
      })
      sendSocketMessage({ data })
    });
  }

  // 回应呼叫
  const answer = (sendData: CallingMessage, action: 'Hangup' | 'Connect') => {
    if(action === 'Connect') channel.joinCallingChannel(sendData.channelID);
    const data = JSON.stringify({
      fromID: user.value.id,
      fromName: user.value.nickname,
      toID: sendData.fromID,
      toName: sendData.fromName,
      channelID: sendData.channelID,
      msgMainFlag: 'CallAnswer',
      msgSubFlag: action
    })
    sendSocketMessage({ data })
  }

  // 挂断
  const hangup = () => {
    const data = JSON.stringify({
      fromID: user.value.id,
      fromName: user.value.nickname,
      toID: targetUser.value.id,
      toNmame: targetUser.value.nickname,
      channelID: channel.current.value.channel_id,
      msgMainFlag: 'CallOffer',
      msgSubFlag: 'Hangup'
    })
  }

  const callingState$ = new Subject()

  return {
    send,
    targetUser,
    setTarget,
    callingState$
  }
}